class obsidianpermutation:
    def __init__(self, block_size=32):
        self._block_size = block_size
        self._perm_table = list(range(block_size * 8))
        self._inv_perm_table = list(range(block_size * 8))
    
    def generate_permutation(self, seed):
        state = seed
        total_bits = self._block_size * 8
        perm = list(range(total_bits))
        
        for i in range(total_bits - 1, 0, -1):
            state = self._prng_step(state)
            j = state % (i + 1)
            perm[i], perm[j] = perm[j], perm[i]
        
        self._perm_table = perm
        self._inv_perm_table = [0] * total_bits
        for i, p in enumerate(perm):
            self._inv_perm_table[p] = i
    
    def _prng_step(self, state):
        state ^= (state << 21) & 0xffffffffffffffff
        state ^= (state >> 35) & 0xffffffffffffffff
        state ^= (state << 4) & 0xffffffffffffffff
        return (state * 0x2545f4914f6cdd1d) & 0xffffffffffffffff
    
    def permute(self, data):
        if len(data) != self._block_size:
            raise ValueError(f"data must be {self._block_size} bytes")
        
        bits = []
        for byte in data:
            for i in range(7, -1, -1):
                bits.append((byte >> i) & 1)
        
        permuted_bits = [bits[self._perm_table[i]] for i in range(len(bits))]
        
        result = bytearray(self._block_size)
        for i in range(self._block_size):
            byte = 0
            for j in range(8):
                byte = (byte << 1) | permuted_bits[i * 8 + j]
            result[i] = byte
        
        return bytes(result)
    
    def inverse_permute(self, data):
        if len(data) != self._block_size:
            raise ValueError(f"data must be {self._block_size} bytes")
        
        bits = []
        for byte in data:
            for i in range(7, -1, -1):
                bits.append((byte >> i) & 1)
        
        permuted_bits = [bits[self._inv_perm_table[i]] for i in range(len(bits))]
        
        result = bytearray(self._block_size)
        for i in range(self._block_size):
            byte = 0
            for j in range(8):
                byte = (byte << 1) | permuted_bits[i * 8 + j]
            result[i] = byte
        
        return bytes(result)



import os
import hashlib


class obsidianvm:
    op_nop = 0x00
    op_xor = 0x01
    op_add = 0x02
    op_sub = 0x03
    op_mul = 0x04
    op_div = 0x05
    op_mod = 0x06
    op_and = 0x07
    op_or = 0x08
    op_not = 0x09
    op_rol = 0x0a
    op_ror = 0x0b
    op_shl = 0x0c
    op_shr = 0x0d
    op_sbox = 0x0e
    op_perm = 0x0f
    op_mix = 0x10
    op_jmp = 0x20
    op_jz = 0x21
    op_jnz = 0x22
    op_jg = 0x23
    op_jl = 0x24
    op_je = 0x25
    op_jne = 0x26
    op_call = 0x30
    op_ret = 0x31
    op_push = 0x40
    op_pop = 0x41
    op_pushf = 0x42
    op_popf = 0x43
    op_load = 0x50
    op_store = 0x51
    op_loadb = 0x52
    op_storeb = 0x53
    op_loadw = 0x54
    op_storew = 0x55
    op_mov = 0x60
    op_movi = 0x61
    op_cmp = 0x62
    op_test = 0x63
    op_inc = 0x64
    op_dec = 0x65
    op_neg = 0x66
    op_swap = 0x67
    op_xchg = 0x68
    op_lea = 0x69
    op_int = 0x70
    op_iret = 0x71
    op_cli = 0x72
    op_sti = 0x73
    op_in = 0x80
    op_out = 0x81
    op_syscall = 0x90
    op_sysret = 0x91
    op_hash = 0xa0
    op_encrypt = 0xa1
    op_decrypt = 0xa2
    op_rand = 0xa3
    op_halt = 0xff
    
    flag_zero = 0x01
    flag_carry = 0x02
    flag_sign = 0x04
    flag_overflow = 0x08
    flag_interrupt = 0x10
    
    int_timer = 0x00
    int_keyboard = 0x01
    int_disk = 0x02
    int_crypto = 0x03
    
    syscall_read = 0x00
    syscall_write = 0x01
    syscall_open = 0x02
    syscall_close = 0x03
    syscall_malloc = 0x04
    syscall_free = 0x05
    syscall_exit = 0x06
    syscall_getpid = 0x07
    syscall_time = 0x08
    syscall_random = 0x09
    
    def __init__(self, memory_size=1048576, num_registers=32):
        self._memory_size = memory_size
        self._memory = bytearray(memory_size)
        self._num_registers = num_registers
        self._registers = [0] * num_registers
        self._pc = 0
        self._sp = memory_size - 0x10000
        self._bp = self._sp
        self._flags = 0
        self._running = False
        self._halted = False
        self._interrupt_enabled = True
        self._interrupt_vector = [0] * 256
        self._interrupt_pending = []
        self._call_stack = []
        self._max_instructions = 100000000
        self._instruction_count = 0
        self._sbox = self._generate_sbox()
        self._inv_sbox = self._generate_inv_sbox()
        self._perm_table = self._generate_perm()
        self._inv_perm_table = self._generate_inv_perm()
        self._heap_start = 0x100000
        self._heap_end = memory_size - 0x20000
        self._heap_ptr = self._heap_start
        self._allocations = {}
        self._io_ports = {}
        self._devices = {}
        self._timer_counter = 0
        self._timer_interval = 1000
        self._stdin_buffer = bytearray()
        self._stdout_buffer = bytearray()
        self._files = {}
        self._next_fd = 3
        self._pid = os.getpid() & 0xffff
        self._entropy_pool = bytearray(256)
        self._entropy_index = 0
        self._init_entropy()
    
    def _init_entropy(self):
        import time
        seed = int(time.time() * 1000000) ^ id(self)
        for i in range(256):
            seed = (seed * 1103515245 + 12345) & 0xffffffff
            self._entropy_pool[i] = seed & 0xff
    
    def _generate_sbox(self):
        sbox = list(range(256))
        state = 0x5a3c9f7e2d1b4a8c
        for i in range(255, 0, -1):
            state ^= (state << 13) & 0xffffffffffffffff
            state ^= (state >> 7) & 0xffffffffffffffff
            state ^= (state << 17) & 0xffffffffffffffff
            j = state % (i + 1)
            sbox[i], sbox[j] = sbox[j], sbox[i]
        return sbox
    
    def _generate_inv_sbox(self):
        inv = [0] * 256
        for i, v in enumerate(self._sbox):
            inv[v] = i
        return inv
    
    def _generate_perm(self):
        perm = list(range(8))
        state = 0xa5c3e1f7b9d2840c
        for i in range(7, 0, -1):
            state ^= (state << 21) & 0xffffffffffffffff
            state ^= (state >> 35) & 0xffffffffffffffff
            j = state % (i + 1)
            perm[i], perm[j] = perm[j], perm[i]
        return perm
    
    def _generate_inv_perm(self):
        inv = [0] * 8
        for i, v in enumerate(self._perm_table):
            inv[v] = i
        return inv
    
    def reset(self):
        self._memory = bytearray(self._memory_size)
        self._registers = [0] * self._num_registers
        self._pc = 0
        self._sp = self._memory_size - 0x10000
        self._bp = self._sp
        self._flags = 0
        self._running = False
        self._halted = False
        self._interrupt_enabled = True
        self._call_stack = []
        self._instruction_count = 0
        self._heap_ptr = self._heap_start
        self._allocations = {}
        self._stdout_buffer = bytearray()
        self._stdin_buffer = bytearray()
    
    def load_program(self, program, offset=0):
        if offset + len(program) > self._memory_size:
            raise MemoryError("program too large")
        for i, byte in enumerate(program):
            self._memory[offset + i] = byte
    
    def load_data(self, data, offset):
        if offset + len(data) > self._memory_size:
            raise MemoryError("data too large")
        for i, byte in enumerate(data):
            self._memory[offset + i] = byte
    
    def get_data(self, offset, length):
        if offset + length > self._memory_size:
            raise MemoryError("read beyond bounds")
        return bytes(self._memory[offset:offset + length])
    
    def set_register(self, reg, value):
        if 0 <= reg < self._num_registers:
            self._registers[reg] = value & 0xffffffff
    
    def get_register(self, reg):
        if 0 <= reg < self._num_registers:
            return self._registers[reg]
        return 0
    
    def _read_byte(self, addr):
        if 0 <= addr < self._memory_size:
            return self._memory[addr]
        return 0
    
    def _write_byte(self, addr, value):
        if 0 <= addr < self._memory_size:
            self._memory[addr] = value & 0xff
    
    def _read_word(self, addr):
        low = self._read_byte(addr)
        high = self._read_byte(addr + 1)
        return (high << 8) | low
    
    def _write_word(self, addr, value):
        self._write_byte(addr, value & 0xff)
        self._write_byte(addr + 1, (value >> 8) & 0xff)
    
    def _read_dword(self, addr):
        b0 = self._read_byte(addr)
        b1 = self._read_byte(addr + 1)
        b2 = self._read_byte(addr + 2)
        b3 = self._read_byte(addr + 3)
        return b0 | (b1 << 8) | (b2 << 16) | (b3 << 24)
    
    def _write_dword(self, addr, value):
        self._write_byte(addr, value & 0xff)
        self._write_byte(addr + 1, (value >> 8) & 0xff)
        self._write_byte(addr + 2, (value >> 16) & 0xff)
        self._write_byte(addr + 3, (value >> 24) & 0xff)
    
    def _fetch(self):
        if self._pc >= self._memory_size:
            return self.op_halt
        opcode = self._memory[self._pc]
        self._pc += 1
        return opcode
    
    def _fetch_byte(self):
        return self._fetch()
    
    def _fetch_word(self):
        low = self._fetch_byte()
        high = self._fetch_byte()
        return (high << 8) | low
    
    def _fetch_dword(self):
        b0 = self._fetch_byte()
        b1 = self._fetch_byte()
        b2 = self._fetch_byte()
        b3 = self._fetch_byte()
        return b0 | (b1 << 8) | (b2 << 16) | (b3 << 24)
    
    def _push_byte(self, value):
        self._sp -= 1
        self._write_byte(self._sp, value)
    
    def _pop_byte(self):
        value = self._read_byte(self._sp)
        self._sp += 1
        return value
    
    def _push_word(self, value):
        self._sp -= 2
        self._write_word(self._sp, value)
    
    def _pop_word(self):
        value = self._read_word(self._sp)
        self._sp += 2
        return value
    
    def _push_dword(self, value):
        self._sp -= 4
        self._write_dword(self._sp, value)
    
    def _pop_dword(self):
        value = self._read_dword(self._sp)
        self._sp += 4
        return value
    
    def _set_flag(self, flag, value):
        if value:
            self._flags |= flag
        else:
            self._flags &= ~flag
    
    def _get_flag(self, flag):
        return (self._flags & flag) != 0
    
    def _update_flags(self, result, is_signed=False):
        self._set_flag(self.flag_zero, result == 0)
        self._set_flag(self.flag_sign, (result & 0x80000000) != 0)
        self._set_flag(self.flag_carry, result > 0xffffffff or result < 0)
        if is_signed:
            self._set_flag(self.flag_overflow, result > 0x7fffffff or result < -0x80000000)
    
    def _rol8(self, val, shift):
        shift %= 8
        return ((val << shift) | (val >> (8 - shift))) & 0xff
    
    def _ror8(self, val, shift):
        shift %= 8
        return ((val >> shift) | (val << (8 - shift))) & 0xff
    
    def _rol32(self, val, shift):
        shift %= 32
        return ((val << shift) | (val >> (32 - shift))) & 0xffffffff
    
    def _ror32(self, val, shift):
        shift %= 32
        return ((val >> shift) | (val << (32 - shift))) & 0xffffffff
    
    def _permute_byte(self, val):
        bits = [(val >> i) & 1 for i in range(8)]
        permuted = [bits[self._perm_table[i]] for i in range(8)]
        result = 0
        for i in range(8):
            result |= permuted[i] << i
        return result
    
    def _inv_permute_byte(self, val):
        bits = [(val >> i) & 1 for i in range(8)]
        permuted = [bits[self._inv_perm_table[i]] for i in range(8)]
        result = 0
        for i in range(8):
            result |= permuted[i] << i
        return result
    
    def _malloc(self, size):
        if self._heap_ptr + size > self._heap_end:
            return 0
        addr = self._heap_ptr
        self._heap_ptr += size
        self._allocations[addr] = size
        return addr
    
    def _free(self, addr):
        if addr in self._allocations:
            del self._allocations[addr]
    
    def _get_random(self):
        idx = self._entropy_index
        self._entropy_index = (self._entropy_index + 1) % 256
        
        val = self._entropy_pool[idx]
        self._entropy_pool[idx] = (val * 1103515245 + 12345) & 0xff
        
        return val
    
    def _get_random_dword(self):
        b0 = self._get_random()
        b1 = self._get_random()
        b2 = self._get_random()
        b3 = self._get_random()
        return b0 | (b1 << 8) | (b2 << 16) | (b3 << 24)
    
    def _hash_block(self, addr, length):
        data = self.get_data(addr, length)
        h = hashlib.sha256(data).digest()
        return int.from_bytes(h[:4], "little")
    
    def _trigger_interrupt(self, int_num):
        if self._interrupt_enabled:
            self._interrupt_pending.append(int_num)
    
    def _handle_interrupts(self):
        if not self._interrupt_enabled or not self._interrupt_pending:
            return
        
        int_num = self._interrupt_pending.pop(0)
        handler = self._interrupt_vector[int_num]
        
        if handler != 0:
            self._push_dword(self._flags)
            self._push_dword(self._pc)
            self._interrupt_enabled = False
            self._pc = handler
    
    def _handle_syscall(self):
        syscall_num = self._registers[0]
        
        if syscall_num == self.syscall_read:
            fd = self._registers[1]
            buf = self._registers[2]
            count = self._registers[3]
            
            if fd == 0 and self._stdin_buffer:
                data = self._stdin_buffer[:count]
                self._stdin_buffer = self._stdin_buffer[count:]
                for i, b in enumerate(data):
                    self._write_byte(buf + i, b)
                self._registers[0] = len(data)
            else:
                self._registers[0] = 0
        
        elif syscall_num == self.syscall_write:
            fd = self._registers[1]
            buf = self._registers[2]
            count = self._registers[3]
            
            if fd == 1:
                data = self.get_data(buf, count)
                self._stdout_buffer.extend(data)
                self._registers[0] = count
            else:
                self._registers[0] = 0
        
        elif syscall_num == self.syscall_malloc:
            size = self._registers[1]
            addr = self._malloc(size)
            self._registers[0] = addr
        
        elif syscall_num == self.syscall_free:
            addr = self._registers[1]
            self._free(addr)
            self._registers[0] = 0
        
        elif syscall_num == self.syscall_exit:
            self._running = False
            self._halted = True
        
        elif syscall_num == self.syscall_getpid:
            self._registers[0] = self._pid
        
        elif syscall_num == self.syscall_time:
            import time
            self._registers[0] = int(time.time()) & 0xffffffff
        
        elif syscall_num == self.syscall_random:
            self._registers[0] = self._get_random_dword()
        
        else:
            self._registers[0] = 0xffffffff
    
    def _execute(self, opcode):
        if opcode == self.op_nop:
            pass
        
        elif opcode == self.op_xor:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            result = self._registers[dst] ^ self._registers[src]
            self._registers[dst] = result & 0xffffffff
            self._update_flags(result)
        
        elif opcode == self.op_add:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            result = self._registers[dst] + self._registers[src]
            self._registers[dst] = result & 0xffffffff
            self._update_flags(result)
        
        elif opcode == self.op_sub:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            result = self._registers[dst] - self._registers[src]
            self._registers[dst] = result & 0xffffffff
            self._update_flags(result)
        
        elif opcode == self.op_mul:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            result = self._registers[dst] * self._registers[src]
            self._registers[dst] = result & 0xffffffff
            self._update_flags(result)
        
        elif opcode == self.op_div:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            if self._registers[src] != 0:
                result = self._registers[dst] // self._registers[src]
                self._registers[dst] = result & 0xffffffff
            self._update_flags(self._registers[dst])
        
        elif opcode == self.op_mod:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            if self._registers[src] != 0:
                result = self._registers[dst] % self._registers[src]
                self._registers[dst] = result & 0xffffffff
            self._update_flags(self._registers[dst])
        
        elif opcode == self.op_and:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            result = self._registers[dst] & self._registers[src]
            self._registers[dst] = result
            self._update_flags(result)
        
        elif opcode == self.op_or:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            result = self._registers[dst] | self._registers[src]
            self._registers[dst] = result
            self._update_flags(result)
        
        elif opcode == self.op_not:
            reg = self._fetch_byte() & 0x1f
            result = ~self._registers[reg] & 0xffffffff
            self._registers[reg] = result
            self._update_flags(result)
        
        elif opcode == self.op_rol:
            reg = self._fetch_byte() & 0x1f
            shift = self._fetch_byte()
            self._registers[reg] = self._rol32(self._registers[reg], shift)
        
        elif opcode == self.op_ror:
            reg = self._fetch_byte() & 0x1f
            shift = self._fetch_byte()
            self._registers[reg] = self._ror32(self._registers[reg], shift)
        
        elif opcode == self.op_shl:
            reg = self._fetch_byte() & 0x1f
            shift = self._fetch_byte()
            result = (self._registers[reg] << shift) & 0xffffffff
            self._registers[reg] = result
            self._update_flags(result)
        
        elif opcode == self.op_shr:
            reg = self._fetch_byte() & 0x1f
            shift = self._fetch_byte()
            result = self._registers[reg] >> shift
            self._registers[reg] = result
            self._update_flags(result)
        
        elif opcode == self.op_sbox:
            reg = self._fetch_byte() & 0x1f
            val = self._registers[reg] & 0xff
            self._registers[reg] = (self._registers[reg] & 0xffffff00) | self._sbox[val]
        
        elif opcode == self.op_perm:
            reg = self._fetch_byte() & 0x1f
            val = self._registers[reg] & 0xff
            self._registers[reg] = (self._registers[reg] & 0xffffff00) | self._permute_byte(val)
        
        elif opcode == self.op_mix:
            r1 = self._fetch_byte() & 0x1f
            r2 = self._fetch_byte() & 0x1f
            a = self._registers[r1] & 0xff
            b = self._registers[r2] & 0xff
            a = self._rol8(a, 3)
            b = self._ror8(b, 5)
            a ^= b
            b = (b + a) & 0xff
            self._registers[r1] = (self._registers[r1] & 0xffffff00) | a
            self._registers[r2] = (self._registers[r2] & 0xffffff00) | b
        
        elif opcode == self.op_jmp:
            addr = self._fetch_dword()
            self._pc = addr
        
        elif opcode == self.op_jz:
            addr = self._fetch_dword()
            if self._get_flag(self.flag_zero):
                self._pc = addr
        
        elif opcode == self.op_jnz:
            addr = self._fetch_dword()
            if not self._get_flag(self.flag_zero):
                self._pc = addr
        
        elif opcode == self.op_jg:
            addr = self._fetch_dword()
            if not self._get_flag(self.flag_zero) and not self._get_flag(self.flag_sign):
                self._pc = addr
        
        elif opcode == self.op_jl:
            addr = self._fetch_dword()
            if self._get_flag(self.flag_sign):
                self._pc = addr
        
        elif opcode == self.op_je:
            addr = self._fetch_dword()
            if self._get_flag(self.flag_zero):
                self._pc = addr
        
        elif opcode == self.op_jne:
            addr = self._fetch_dword()
            if not self._get_flag(self.flag_zero):
                self._pc = addr
        
        elif opcode == self.op_call:
            addr = self._fetch_dword()
            self._push_dword(self._pc)
            self._call_stack.append(self._pc)
            self._pc = addr
        
        elif opcode == self.op_ret:
            if self._call_stack:
                self._call_stack.pop()
            self._pc = self._pop_dword()
        
        elif opcode == self.op_push:
            reg = self._fetch_byte() & 0x1f
            self._push_dword(self._registers[reg])
        
        elif opcode == self.op_pop:
            reg = self._fetch_byte() & 0x1f
            self._registers[reg] = self._pop_dword()
        
        elif opcode == self.op_pushf:
            self._push_dword(self._flags)
        
        elif opcode == self.op_popf:
            self._flags = self._pop_dword() & 0x1f
        
        elif opcode == self.op_load:
            reg = self._fetch_byte() & 0x1f
            addr = self._fetch_dword()
            self._registers[reg] = self._read_dword(addr)
        
        elif opcode == self.op_store:
            reg = self._fetch_byte() & 0x1f
            addr = self._fetch_dword()
            self._write_dword(addr, self._registers[reg])
        
        elif opcode == self.op_loadb:
            reg = self._fetch_byte() & 0x1f
            addr = self._fetch_dword()
            self._registers[reg] = self._read_byte(addr)
        
        elif opcode == self.op_storeb:
            reg = self._fetch_byte() & 0x1f
            addr = self._fetch_dword()
            self._write_byte(addr, self._registers[reg] & 0xff)
        
        elif opcode == self.op_loadw:
            reg = self._fetch_byte() & 0x1f
            addr = self._fetch_dword()
            self._registers[reg] = self._read_word(addr)
        
        elif opcode == self.op_storew:
            reg = self._fetch_byte() & 0x1f
            addr = self._fetch_dword()
            self._write_word(addr, self._registers[reg] & 0xffff)
        
        elif opcode == self.op_mov:
            dst = self._fetch_byte() & 0x1f
            src = self._fetch_byte() & 0x1f
            self._registers[dst] = self._registers[src]
        
        elif opcode == self.op_movi:
            reg = self._fetch_byte() & 0x1f
            imm = self._fetch_dword()
            self._registers[reg] = imm
        
        elif opcode == self.op_cmp:
            r1 = self._fetch_byte() & 0x1f
            r2 = self._fetch_byte() & 0x1f
            result = self._registers[r1] - self._registers[r2]
            self._update_flags(result, is_signed=True)
        
        elif opcode == self.op_test:
            r1 = self._fetch_byte() & 0x1f
            r2 = self._fetch_byte() & 0x1f
            result = self._registers[r1] & self._registers[r2]
            self._update_flags(result)
        
        elif opcode == self.op_inc:
            reg = self._fetch_byte() & 0x1f
            result = self._registers[reg] + 1
            self._registers[reg] = result & 0xffffffff
            self._update_flags(result)
        
        elif opcode == self.op_dec:
            reg = self._fetch_byte() & 0x1f
            result = self._registers[reg] - 1
            self._registers[reg] = result & 0xffffffff
            self._update_flags(result)
        
        elif opcode == self.op_neg:
            reg = self._fetch_byte() & 0x1f
            result = -self._registers[reg]
            self._registers[reg] = result & 0xffffffff
            self._update_flags(result)
        
        elif opcode == self.op_swap:
            r1 = self._fetch_byte() & 0x1f
            r2 = self._fetch_byte() & 0x1f
            self._registers[r1], self._registers[r2] = self._registers[r2], self._registers[r1]
        
        elif opcode == self.op_xchg:
            reg = self._fetch_byte() & 0x1f
            addr = self._fetch_dword()
            mem_val = self._read_dword(addr)
            self._write_dword(addr, self._registers[reg])
            self._registers[reg] = mem_val
        
        elif opcode == self.op_lea:
            reg = self._fetch_byte() & 0x1f
            addr = self._fetch_dword()
            self._registers[reg] = addr
        
        elif opcode == self.op_int:
            int_num = self._fetch_byte()
            self._trigger_interrupt(int_num)
        
        elif opcode == self.op_iret:
            self._pc = self._pop_dword()
            self._flags = self._pop_dword()
            self._interrupt_enabled = True
        
        elif opcode == self.op_cli:
            self._interrupt_enabled = False
        
        elif opcode == self.op_sti:
            self._interrupt_enabled = True
        
        elif opcode == self.op_in:
            reg = self._fetch_byte() & 0x1f
            port = self._fetch_word()
            self._registers[reg] = self._io_ports.get(port, 0)
        
        elif opcode == self.op_out:
            port = self._fetch_word()
            reg = self._fetch_byte() & 0x1f
            self._io_ports[port] = self._registers[reg]
        
        elif opcode == self.op_syscall:
            self._handle_syscall()
        
        elif opcode == self.op_sysret:
            self._pc = self._pop_dword()
        
        elif opcode == self.op_hash:
            reg = self._fetch_byte() & 0x1f
            addr = self._registers[reg]
            length = self._registers[(reg + 1) & 0x1f]
            self._registers[reg] = self._hash_block(addr, length)
        
        elif opcode == self.op_encrypt:
            data_reg = self._fetch_byte() & 0x1f
            key_reg = self._fetch_byte() & 0x1f
            len_reg = self._fetch_byte() & 0x1f
            data_addr = self._registers[data_reg]
            key_addr = self._registers[key_reg]
            length = self._registers[len_reg]
            
            for i in range(length):
                d = self._read_byte(data_addr + i)
                k = self._read_byte(key_addr + (i % 32))
                d ^= k
                d = self._sbox[d]
                d = self._rol8(d, 3)
                self._write_byte(data_addr + i, d)
        
        elif opcode == self.op_decrypt:
            data_reg = self._fetch_byte() & 0x1f
            key_reg = self._fetch_byte() & 0x1f
            len_reg = self._fetch_byte() & 0x1f
            data_addr = self._registers[data_reg]
            key_addr = self._registers[key_reg]
            length = self._registers[len_reg]
            
            for i in range(length):
                d = self._read_byte(data_addr + i)
                k = self._read_byte(key_addr + (i % 32))
                d = self._ror8(d, 3)
                d = self._inv_sbox[d]
                d ^= k
                self._write_byte(data_addr + i, d)
        
        elif opcode == self.op_rand:
            reg = self._fetch_byte() & 0x1f
            self._registers[reg] = self._get_random_dword()
        
        elif opcode == self.op_halt:
            self._running = False
            self._halted = True
        
        else:
            pass
    
    def run(self, start_address=0):
        self._pc = start_address
        self._running = True
        self._halted = False
        self._instruction_count = 0
        
        while self._running and self._instruction_count < self._max_instructions:
            self._handle_interrupts()
            
            opcode = self._fetch()
            self._execute(opcode)
            self._instruction_count += 1
            
            self._timer_counter += 1
            if self._timer_counter >= self._timer_interval:
                self._timer_counter = 0
                self._trigger_interrupt(self.int_timer)
        
        return self._registers[0]
    
    def step(self):
        if self._pc >= self._memory_size:
            self._running = False
            return False
        
        self._handle_interrupts()
        opcode = self._fetch()
        self._execute(opcode)
        return self._running
    
    def is_running(self):
        return self._running
    
    def is_halted(self):
        return self._halted
    
    def get_pc(self):
        return self._pc
    
    def get_sp(self):
        return self._sp
    
    def get_bp(self):
        return self._bp
    
    def get_flags(self):
        return self._flags
    
    def get_stdout(self):
        return bytes(self._stdout_buffer)
    
    def set_stdin(self, data):
        if isinstance(data, str):
            data = data.encode()
        self._stdin_buffer.extend(data)
    
    def set_interrupt_vector(self, int_num, handler_addr):
        if 0 <= int_num < 256:
            self._interrupt_vector[int_num] = handler_addr
    
    def add_device(self, port, device):
        self._devices[port] = device
    
    def create_encryption_program(self, data_offset, key_offset, output_offset, length):
        program = []
        
        program.extend([self.op_movi, 0])
        program.extend(data_offset.to_bytes(4, "little"))
        
        program.extend([self.op_movi, 1])
        program.extend(key_offset.to_bytes(4, "little"))
        
        program.extend([self.op_movi, 2])
        program.extend(length.to_bytes(4, "little"))
        
        program.extend([self.op_encrypt, 0, 1, 2])
        
        for i in range(length):
            program.extend([self.op_loadb, 3])
            program.extend((data_offset + i).to_bytes(4, "little"))
            program.extend([self.op_storeb, 3])
            program.extend((output_offset + i).to_bytes(4, "little"))
        
        program.append(self.op_halt)
        return bytes(program)
    
    def create_decryption_program(self, data_offset, key_offset, output_offset, length):
        program = []
        
        for i in range(length):
            program.extend([self.op_loadb, 0])
            program.extend((data_offset + i).to_bytes(4, "little"))
            program.extend([self.op_storeb, 0])
            program.extend((output_offset + i).to_bytes(4, "little"))
        
        program.extend([self.op_movi, 0])
        program.extend(output_offset.to_bytes(4, "little"))
        
        program.extend([self.op_movi, 1])
        program.extend(key_offset.to_bytes(4, "little"))
        
        program.extend([self.op_movi, 2])
        program.extend(length.to_bytes(4, "little"))
        
        program.extend([self.op_decrypt, 0, 1, 2])
        
        program.append(self.op_halt)
        return bytes(program)
    
    def assemble(self, source):
        instructions = {
            "nop": self.op_nop,
            "xor": self.op_xor,
            "add": self.op_add,
            "sub": self.op_sub,
            "mul": self.op_mul,
            "div": self.op_div,
            "mod": self.op_mod,
            "and": self.op_and,
            "or": self.op_or,
            "not": self.op_not,
            "rol": self.op_rol,
            "ror": self.op_ror,
            "shl": self.op_shl,
            "shr": self.op_shr,
            "sbox": self.op_sbox,
            "perm": self.op_perm,
            "mix": self.op_mix,
            "jmp": self.op_jmp,
            "jz": self.op_jz,
            "jnz": self.op_jnz,
            "jg": self.op_jg,
            "jl": self.op_jl,
            "je": self.op_je,
            "jne": self.op_jne,
            "call": self.op_call,
            "ret": self.op_ret,
            "push": self.op_push,
            "pop": self.op_pop,
            "load": self.op_load,
            "store": self.op_store,
            "mov": self.op_mov,
            "movi": self.op_movi,
            "cmp": self.op_cmp,
            "inc": self.op_inc,
            "dec": self.op_dec,
            "syscall": self.op_syscall,
            "halt": self.op_halt,
        }
        
        dword_imm_ops = {"movi", "jmp", "jz", "jnz", "jg", "jl", "je", "jne", "call", "load", "store"}
        
        program = []
        labels = {}
        lines = source.strip().split("\n")
        
        offset = 0
        for line in lines:
            line = line.strip()
            if not line or line.startswith(";"):
                continue
            if line.endswith(":"):
                labels[line[:-1]] = offset
            else:
                parts = line.split()
                mnem = parts[0].lower()
                if mnem in instructions:
                    offset += 1
                    for i, arg in enumerate(parts[1:]):
                        arg = arg.rstrip(",")
                        if arg.startswith("r"):
                            offset += 1
                        elif mnem in dword_imm_ops:
                            offset += 4
                        else:
                            offset += 1
        
        for line in lines:
            line = line.strip()
            if not line or line.startswith(";") or line.endswith(":"):
                continue
            
            parts = line.split()
            mnemonic = parts[0].lower()
            
            if mnemonic in instructions:
                program.append(instructions[mnemonic])
                
                for i, arg in enumerate(parts[1:]):
                    arg = arg.rstrip(",")
                    if arg.startswith("r"):
                        program.append(int(arg[1:]))
                    elif arg in labels:
                        addr = labels[arg]
                        program.extend(addr.to_bytes(4, "little"))
                    elif mnemonic in dword_imm_ops and not arg.startswith("r"):
                        if arg.startswith("0x"):
                            val = int(arg, 16)
                        else:
                            val = int(arg)
                        program.extend(val.to_bytes(4, "little"))
                    elif arg.startswith("0x"):
                        val = int(arg, 16)
                        program.append(val & 0xff)
                    else:
                        val = int(arg)
                        program.append(val & 0xff)
        
        return bytes(program)



from .core.cipher import obsidian256cipher
from .core.keygen import keygenerator
from .core.utils import bytes_to_hex, hex_to_bytes
from .core.advanced_cipher import (
    obsidian3300,
    quantumstate,
    chaosengine,
    nonlinearfeedback,
    dynamicsbox,
    contextboundkey,
)
from .vm.virtual_machine import obsidianvm

__version__ = "1.0.0"
__author__ = "mero"
__contact__ = "telegram: @qp4rm"

__all__ = [
    "obsidian256cipher",
    "obsidian3300",
    "keygenerator",
    "obsidianvm",
    "quantumstate",
    "chaosengine",
    "nonlinearfeedback",
    "dynamicsbox",
    "contextboundkey",
    "bytes_to_hex",
    "hex_to_bytes",
]



import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), "..", "src"))

from obsidian256 import (
    obsidian3300,
    quantumstate,
    chaosengine,
    nonlinearfeedback,
    dynamicsbox,
    contextboundkey,
)


def test_quantum_state():
    qs = quantumstate()
    qs.absorb(b"test data")
    output = qs.squeeze(32)
    assert len(output) == 32
    assert output != b"\x00" * 32


def test_chaos_engine():
    ce = chaosengine(0x123456789abcdef0)
    val1 = ce.generate()
    val2 = ce.generate()
    assert val1 != val2


def test_nonlinear_feedback():
    nf = nonlinearfeedback(0x1111111111111111, 0x2222222222222222, 0x3333333333333333)
    val1 = nf.clock()
    val2 = nf.clock()
    assert val1 != val2


def test_dynamic_sbox():
    key = b"test key for sbox generation"
    sbox = dynamicsbox(key)
    
    data = bytes(range(256))
    substituted = sbox.substitute(data)
    restored = sbox.inverse_substitute(substituted)
    
    assert restored == data


def test_context_bound_key():
    cb = contextboundkey()
    context = cb.get_context()
    assert len(context) > 0
    
    master_key = os.urandom(32)
    bound_key, ctx = cb.derive_bound_key(master_key)
    
    assert len(bound_key) == len(master_key)
    assert bound_key != master_key


def test_obsidian3300_basic():
    cipher = obsidian3300()
    
    plaintext = b"test message for 3300-bit encryption"
    ciphertext = cipher.encrypt(plaintext)
    decrypted = cipher.decrypt(ciphertext)
    
    assert decrypted == plaintext


def test_obsidian3300_context_bound():
    cipher = obsidian3300(context_bound=True)
    
    plaintext = b"context bound encryption test"
    ciphertext = cipher.encrypt(plaintext)
    decrypted = cipher.decrypt(ciphertext)
    
    assert decrypted == plaintext
    assert cipher.get_context() is not None


def test_obsidian3300_with_iv():
    cipher = obsidian3300()
    
    plaintext = b"encryption with iv prepended"
    data = cipher.encrypt_with_iv(plaintext)
    
    cipher2 = obsidian3300(key=cipher.get_key())
    decrypted = cipher2.decrypt_with_iv(data)
    
    assert decrypted == plaintext


def test_obsidian3300_various_sizes():
    cipher = obsidian3300()
    
    sizes = [1, 100, 500, 1000]
    for size in sizes:
        plaintext = os.urandom(size)
        ciphertext = cipher.encrypt(plaintext)
        decrypted = cipher.decrypt(ciphertext)
        assert decrypted == plaintext


def test_avalanche_3300():
    key1 = b"\x00" * 413
    key2 = b"\x01" + b"\x00" * 412
    
    cipher1 = obsidian3300(key=key1)
    cipher2 = obsidian3300(key=key2)
    
    cipher2._iv = cipher1._iv
    cipher2._round_keys = cipher2._generate_round_keys()
    
    plaintext = b"\x00" * 413
    ct1 = cipher1.encrypt(plaintext)
    ct2 = cipher2.encrypt(plaintext)
    
    diff_bits = sum(bin(b1 ^ b2).count("1") for b1, b2 in zip(ct1, ct2))
    total_bits = len(ct1) * 8
    
    percentage = (diff_bits / total_bits) * 100
    print(f"3300-bit avalanche: {diff_bits}/{total_bits} ({percentage:.2f}%)")
    
    assert percentage > 30


if __name__ == "__main__":
    test_quantum_state()
    print("test_quantum_state: passed")
    
    test_chaos_engine()
    print("test_chaos_engine: passed")
    
    test_nonlinear_feedback()
    print("test_nonlinear_feedback: passed")
    
    test_dynamic_sbox()
    print("test_dynamic_sbox: passed")
    
    test_context_bound_key()
    print("test_context_bound_key: passed")
    
    test_obsidian3300_basic()
    print("test_obsidian3300_basic: passed")
    
    test_obsidian3300_context_bound()
    print("test_obsidian3300_context_bound: passed")
    
    test_obsidian3300_with_iv()
    print("test_obsidian3300_with_iv: passed")
    
    test_obsidian3300_various_sizes()
    print("test_obsidian3300_various_sizes: passed")
    
    test_avalanche_3300()
    print("test_avalanche_3300: passed")
    
    print("\nall advanced tests passed!")



from .cipher import obsidian256cipher
from .keygen import keygenerator
from .utils import bytes_to_hex, hex_to_bytes
from .sbox import obsidiansbox
from .permutation import obsidianpermutation
from .mixer import obsidianmixer

__all__ = [
    "obsidian256cipher",
    "keygenerator",
    "bytes_to_hex",
    "hex_to_bytes",
    "obsidiansbox",
    "obsidianpermutation",
    "obsidianmixer",
]



import os
import hashlib
import time


class keygenerator:
    def __init__(self, key_size=32, min_rounds=12, max_rounds=20):
        self._key_size = key_size
        self._min_rounds = min_rounds
        self._max_rounds = max_rounds
    
    def generate_master_key(self, password=None, salt=None):
        if password is None:
            return os.urandom(self._key_size)
        
        if isinstance(password, str):
            password = password.encode("utf-8")
        
        if salt is None:
            salt = os.urandom(16)
        elif isinstance(salt, str):
            salt = salt.encode("utf-8")
        
        derived = self._kdf(password, salt, iterations=100000)
        return derived[:self._key_size], salt
    
    def _kdf(self, password, salt, iterations):
        result = password + salt
        for i in range(iterations):
            h = hashlib.sha512()
            h.update(result)
            h.update(i.to_bytes(8, "little"))
            h.update(salt)
            result = h.digest()
        
        return result
    
    def generate_round_keys(self, master_key, num_rounds=None):
        if num_rounds is None:
            num_rounds = self._calculate_rounds(master_key)
        
        round_keys = []
        state = master_key
        
        for r in range(num_rounds):
            round_key = self._derive_round_key(state, r)
            round_keys.append(round_key)
            state = self._evolve_state(state, round_key)
        
        return round_keys
    
    def _calculate_rounds(self, key):
        key_entropy = sum(key) % 256
        base_rounds = self._min_rounds
        additional = (key_entropy * (self._max_rounds - self._min_rounds)) // 256
        return base_rounds + additional
    
    def _derive_round_key(self, state, round_num):
        h1 = hashlib.sha256()
        h1.update(state)
        h1.update(round_num.to_bytes(4, "little"))
        h1.update(bytes([0x4f, 0x42, 0x53, 0x49, 0x44, 0x49, 0x41, 0x4e]))
        derived1 = h1.digest()
        
        h2 = hashlib.sha256()
        h2.update(derived1)
        h2.update((round_num ^ 0x5a5a5a5a).to_bytes(4, "little"))
        h2.update(state[::-1])
        derived2 = h2.digest()
        
        result = bytearray(self._key_size)
        for i in range(self._key_size):
            d1 = derived1[i % len(derived1)]
            d2 = derived2[i % len(derived2)]
            mixed = (d1 + d2 + round_num + i) & 0xff
            mixed ^= ((d1 * d2) ^ (round_num << 3)) & 0xff
            result[i] = mixed
        
        return bytes(result)
    
    def _evolve_state(self, state, round_key):
        h = hashlib.sha256()
        h.update(state)
        h.update(round_key)
        return h.digest()
    
    def get_round_count(self, master_key):
        return self._calculate_rounds(master_key)
    
    def generate_iv(self):
        return os.urandom(self._key_size)
    
    def generate_keypair(self, password=None):
        if password:
            master_key, salt = self.generate_master_key(password)
        else:
            master_key = self.generate_master_key()
            salt = None
        
        iv = self.generate_iv()
        round_keys = self.generate_round_keys(master_key)
        
        return {
            "master_key": master_key,
            "salt": salt,
            "iv": iv,
            "round_keys": round_keys,
            "num_rounds": len(round_keys),
        }



import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), "..", "src"))

from obsidian256 import obsidianvm


def test_vm_registers():
    vm = obsidianvm()
    
    for i in range(16):
        vm.set_register(i, i * 10)
    
    for i in range(16):
        assert vm.get_register(i) == i * 10


def test_vm_memory():
    vm = obsidianvm()
    
    data = bytes(range(256))
    vm.load_data(data, 0x100)
    
    retrieved = vm.get_data(0x100, 256)
    assert retrieved == data


def test_vm_xor():
    vm = obsidianvm()
    vm.set_register(0, 0xaa)
    vm.set_register(1, 0x55)
    
    program = bytes([
        obsidianvm.op_xor, 0, 1,
        obsidianvm.op_halt,
    ])
    
    vm.load_program(program)
    vm.run()
    
    assert vm.get_register(0) == 0xff


def test_vm_add():
    vm = obsidianvm()
    vm.set_register(0, 100)
    vm.set_register(1, 50)
    
    program = bytes([
        obsidianvm.op_add, 0, 1,
        obsidianvm.op_halt,
    ])
    
    vm.load_program(program)
    vm.run()
    
    assert vm.get_register(0) == 150


def test_vm_rotation():
    vm = obsidianvm()
    vm.set_register(0, 0x80000000)
    
    program = bytes([
        obsidianvm.op_rol, 0, 1,
        obsidianvm.op_halt,
    ])
    
    vm.load_program(program)
    vm.run()
    
    assert vm.get_register(0) == 0x01


def test_vm_sbox():
    vm = obsidianvm()
    vm.set_register(0, 0x00)
    
    program = bytes([
        obsidianvm.op_sbox, 0,
        obsidianvm.op_halt,
    ])
    
    vm.load_program(program)
    initial = vm.get_register(0)
    vm.run()
    result = vm.get_register(0)
    
    assert result != initial or initial == result


def test_vm_encryption_program():
    vm = obsidianvm()
    
    data = b"test"
    key = b"0123456789abcdef0123456789abcdef"
    
    data_offset = 0x1000
    key_offset = 0x2000
    output_offset = 0x3000
    
    vm.load_data(data, data_offset)
    vm.load_data(key, key_offset)
    
    program = vm.create_encryption_program(
        data_offset, key_offset, output_offset, len(data)
    )
    
    vm.load_program(program)
    vm.run()
    
    encrypted = vm.get_data(output_offset, len(data))
    assert encrypted != data


def test_vm_reset():
    vm = obsidianvm()
    
    vm.set_register(0, 0xff)
    vm.load_data(b"test", 0x100)
    
    vm.reset()
    
    assert vm.get_register(0) == 0
    assert vm.get_data(0x100, 4) == b"\x00\x00\x00\x00"


if __name__ == "__main__":
    test_vm_registers()
    print("test_vm_registers: passed")
    
    test_vm_memory()
    print("test_vm_memory: passed")
    
    test_vm_xor()
    print("test_vm_xor: passed")
    
    test_vm_add()
    print("test_vm_add: passed")
    
    test_vm_rotation()
    print("test_vm_rotation: passed")
    
    test_vm_sbox()
    print("test_vm_sbox: passed")
    
    test_vm_encryption_program()
    print("test_vm_encryption_program: passed")
    
    test_vm_reset()
    print("test_vm_reset: passed")
    
    print("\nall vm tests passed!")



class obsidianmixer:
    def __init__(self, block_size=32):
        self._block_size = block_size
        self._mix_constants = self._generate_constants()
    
    def _generate_constants(self):
        constants = []
        phi = 0x9e3779b97f4a7c15
        for i in range(self._block_size):
            c = (phi * (i + 1)) & 0xffffffffffffffff
            c = self._mix64(c)
            constants.append(c & 0xff)
        return constants
    
    def _mix64(self, x):
        x ^= x >> 33
        x = (x * 0xff51afd7ed558ccd) & 0xffffffffffffffff
        x ^= x >> 33
        x = (x * 0xc4ceb9fe1a85ec53) & 0xffffffffffffffff
        x ^= x >> 33
        return x
    
    def _rotate_left(self, val, shift, bits=8):
        shift %= bits
        return ((val << shift) | (val >> (bits - shift))) & ((1 << bits) - 1)
    
    def _rotate_right(self, val, shift, bits=8):
        shift %= bits
        return ((val >> shift) | (val << (bits - shift))) & ((1 << bits) - 1)
    
    def mix_forward(self, data, round_key):
        if len(data) != self._block_size:
            raise ValueError(f"data must be {self._block_size} bytes")
        
        result = bytearray(data)
        
        for i in range(self._block_size):
            result[i] ^= self._mix_constants[i]
        
        for i in range(0, self._block_size - 1, 2):
            a, b = result[i], result[i + 1]
            a = self._rotate_left(a, 3)
            b = self._rotate_right(b, 5)
            a ^= b
            b = (b + a) & 0xff
            result[i], result[i + 1] = a, b
        
        for i in range(self._block_size):
            rk = round_key[i % len(round_key)]
            result[i] = (result[i] + rk) & 0xff
            result[i] = self._rotate_left(result[i], rk & 0x07)
        
        for i in range(self._block_size - 1):
            result[i + 1] ^= result[i]
        
        return bytes(result)
    
    def mix_inverse(self, data, round_key):
        if len(data) != self._block_size:
            raise ValueError(f"data must be {self._block_size} bytes")
        
        result = bytearray(data)
        
        for i in range(self._block_size - 2, -1, -1):
            result[i + 1] ^= result[i]
        
        for i in range(self._block_size - 1, -1, -1):
            rk = round_key[i % len(round_key)]
            result[i] = self._rotate_right(result[i], rk & 0x07)
            result[i] = (result[i] - rk) & 0xff
        
        for i in range(self._block_size - 2, -1, -2):
            a, b = result[i], result[i + 1]
            b = (b - a) & 0xff
            a ^= b
            b = self._rotate_left(b, 5)
            a = self._rotate_right(a, 3)
            result[i], result[i + 1] = a, b
        
        for i in range(self._block_size):
            result[i] ^= self._mix_constants[i]
        
        return bytes(result)



class obsidiansbox:
    def __init__(self, seed=None):
        self._forward = list(range(256))
        self._inverse = list(range(256))
        self._generate_sbox(seed)
    
    def _generate_sbox(self, seed):
        if seed is None:
            seed = 0x5a3c9f7e2d1b4a8c
        
        state = seed
        for i in range(255, 0, -1):
            state = self._prng_next(state)
            j = state % (i + 1)
            self._forward[i], self._forward[j] = self._forward[j], self._forward[i]
        
        for i in range(256):
            self._inverse[self._forward[i]] = i
    
    def _prng_next(self, state):
        state ^= (state << 13) & 0xffffffffffffffff
        state ^= (state >> 7) & 0xffffffffffffffff
        state ^= (state << 17) & 0xffffffffffffffff
        state = (state * 0x5851f42d4c957f2d + 1) & 0xffffffffffffffff
        return state
    
    def substitute(self, data):
        if isinstance(data, (bytes, bytearray)):
            return bytes([self._forward[b] for b in data])
        return self._forward[data & 0xff]
    
    def inverse_substitute(self, data):
        if isinstance(data, (bytes, bytearray)):
            return bytes([self._inverse[b] for b in data])
        return self._inverse[data & 0xff]
    
    def get_forward_table(self):
        return self._forward.copy()
    
    def get_inverse_table(self):
        return self._inverse.copy()



def bytes_to_hex(data):
    return data.hex()


def hex_to_bytes(hex_string):
    return bytes.fromhex(hex_string)


def pad_data(data, block_size=32):
    padding_len = block_size - (len(data) % block_size)
    if padding_len == 0:
        padding_len = block_size
    padding = bytes([padding_len] * padding_len)
    return data + padding


def unpad_data(data):
    if len(data) == 0:
        raise ValueError("empty data")
    padding_len = data[-1]
    if padding_len > len(data) or padding_len == 0:
        raise ValueError("invalid padding")
    for i in range(1, padding_len + 1):
        if data[-i] != padding_len:
            raise ValueError("invalid padding")
    return data[:-padding_len]


def xor_bytes(a, b):
    if len(a) != len(b):
        raise ValueError("byte arrays must have equal length")
    return bytes(x ^ y for x, y in zip(a, b))


def rotate_left_byte(val, shift):
    shift %= 8
    return ((val << shift) | (val >> (8 - shift))) & 0xff


def rotate_right_byte(val, shift):
    shift %= 8
    return ((val >> shift) | (val << (8 - shift))) & 0xff


def split_blocks(data, block_size=32):
    if len(data) % block_size != 0:
        raise ValueError(f"data length must be multiple of {block_size}")
    return [data[i:i + block_size] for i in range(0, len(data), block_size)]


def join_blocks(blocks):
    return b"".join(blocks)


def constant_time_compare(a, b):
    if len(a) != len(b):
        return False
    result = 0
    for x, y in zip(a, b):
        result |= x ^ y
    return result == 0



import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), "..", "src"))

from obsidian256 import obsidian256cipher, keygenerator


def test_basic_encrypt_decrypt():
    cipher = obsidian256cipher()
    plaintext = b"test message"
    ciphertext = cipher.encrypt(plaintext)
    decrypted = cipher.decrypt(ciphertext)
    assert decrypted == plaintext


def test_custom_key():
    key = obsidian256cipher.generate_key()
    iv = obsidian256cipher.generate_iv()
    
    cipher = obsidian256cipher(key=key, iv=iv)
    plaintext = b"custom key test"
    ciphertext = cipher.encrypt(plaintext)
    
    cipher2 = obsidian256cipher(key=key, iv=iv)
    decrypted = cipher2.decrypt(ciphertext)
    assert decrypted == plaintext


def test_password_key():
    kg = keygenerator()
    key, salt = kg.generate_master_key(password="test_password")
    
    cipher = obsidian256cipher(key=key)
    plaintext = b"password protected"
    ciphertext = cipher.encrypt(plaintext)
    
    key2, _ = kg.generate_master_key(password="test_password", salt=salt)
    cipher2 = obsidian256cipher(key=key2, iv=cipher.get_iv())
    decrypted = cipher2.decrypt(ciphertext)
    assert decrypted == plaintext


def test_various_sizes():
    cipher = obsidian256cipher()
    
    sizes = [1, 16, 32, 64, 100, 256, 1000]
    for size in sizes:
        plaintext = bytes([i % 256 for i in range(size)])
        ciphertext = cipher.encrypt(plaintext)
        decrypted = cipher.decrypt(ciphertext)
        assert decrypted == plaintext


def test_string_input():
    cipher = obsidian256cipher()
    plaintext = "unicode string: hello world"
    ciphertext = cipher.encrypt(plaintext)
    decrypted = cipher.decrypt(ciphertext)
    assert decrypted == plaintext.encode("utf-8")


def test_round_count():
    for _ in range(10):
        key = obsidian256cipher.generate_key()
        cipher = obsidian256cipher(key=key)
        rounds = cipher.get_round_count()
        assert 12 <= rounds <= 20


def test_avalanche():
    key1 = b"\x00" * 32
    key2 = b"\x01" + b"\x00" * 31
    iv = b"\x00" * 32
    
    cipher1 = obsidian256cipher(key=key1, iv=iv)
    cipher2 = obsidian256cipher(key=key2, iv=iv)
    
    plaintext = b"\x00" * 32
    ct1 = cipher1.encrypt(plaintext)
    ct2 = cipher2.encrypt(plaintext)
    
    diff_bits = sum(bin(b1 ^ b2).count("1") for b1, b2 in zip(ct1, ct2))
    total_bits = len(ct1) * 8
    
    assert diff_bits > total_bits * 0.3


if __name__ == "__main__":
    test_basic_encrypt_decrypt()
    print("test_basic_encrypt_decrypt: passed")
    
    test_custom_key()
    print("test_custom_key: passed")
    
    test_password_key()
    print("test_password_key: passed")
    
    test_various_sizes()
    print("test_various_sizes: passed")
    
    test_string_input()
    print("test_string_input: passed")
    
    test_round_count()
    print("test_round_count: passed")
    
    test_avalanche()
    print("test_avalanche: passed")
    
    print("\nall tests passed!")



from setuptools import setup, find_packages, Extension
import os
import sys

with open("readme.md", "r", encoding="utf-8") as f:
    long_description = f.read()

native_sources = []
ext_modules = []

native_dir = os.path.join("src", "obsidian256", "native")
if os.path.exists(native_dir):
    c_sources = [os.path.join(native_dir, f) for f in os.listdir(native_dir) if f.endswith(".c")]
    if c_sources:
        ext_modules.append(
            Extension(
                "obsidian256.native.obsidian_core",
                sources=c_sources,
                extra_compile_args=["-O3", "-fPIC"] if sys.platform != "win32" else ["/O2"],
            )
        )

setup(
    name="obsidian256",
    version="1.0.0",
    author="mero",
    author_email="",
    description="obsidian-256 advanced encryption algorithm with 3300-bit support",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://t.me/qp4rm",
    project_urls={
        "telegram": "https://t.me/qp4rm",
    },
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    ext_modules=ext_modules if ext_modules else None,
    include_package_data=True,
    package_data={
        "obsidian256": ["native/*.c", "native/*.cpp", "native/*.h"],
    },
    classifiers=[
        "development status :: 5 - production/stable",
        "intended audience :: developers",
        "license :: osi approved :: mit license",
        "operating system :: os independent",
        "programming language :: python :: 3",
        "programming language :: python :: 3.7",
        "programming language :: python :: 3.8",
        "programming language :: python :: 3.9",
        "programming language :: python :: 3.10",
        "programming language :: python :: 3.11",
        "programming language :: python :: 3.12",
        "programming language :: c",
        "programming language :: c++",
        "topic :: security :: cryptography",
    ],
    python_requires=">=3.7",
    install_requires=[],
    extras_require={
        "dev": [
            "pytest>=6.0",
            "build",
            "twine",
        ],
    },
)



import os
import time
import hashlib
import struct
import platform
import sys


class quantumstate:
    phi = 0x9e3779b97f4a7c15
    sqrt2 = 0x6a09e667bb67ae85
    sqrt3 = 0xb7e151628aed2a6a
    pi = 0x243f6a8885a308d3
    e = 0x2b7e151628aed2a6
    
    def __init__(self):
        self.state = [
            self.phi, self.sqrt2, self.sqrt3, self.pi,
            self.e, 0xc4ceb9fe1a85ec53, 0xff51afd7ed558ccd, 0x62a9d9ed799705f5
        ]
    
    def _rotl(self, x, k):
        k &= 63
        return ((x << k) | (x >> (64 - k))) & 0xffffffffffffffff
    
    def _rotr(self, x, k):
        k &= 63
        return ((x >> k) | (x << (64 - k))) & 0xffffffffffffffff
    
    def absorb(self, data):
        if isinstance(data, str):
            data = data.encode()
        for i, b in enumerate(data):
            self.state[i % 8] ^= b << ((i % 8) * 8)
            self.state[i % 8] &= 0xffffffffffffffff
            self.permute()
    
    def permute(self):
        for r in range(24):
            t = 0
            for i in range(8):
                t ^= self.state[i]
            
            for i in range(8):
                self.state[i] ^= self._rotl(t, 7 + i)
                self.state[i] = (self.state[i] + self.state[(i + 1) % 8]) & 0xffffffffffffffff
                self.state[i] = self._rotl(self.state[i], 17 + r)
                self.state[i] = (self.state[i] * self.phi) & 0xffffffffffffffff
                self.state[i] ^= self._rotr(self.state[i], 29)
            
            theta = 0
            for i in range(8):
                theta ^= self._rotl(self.state[i], i * 8)
            for i in range(8):
                self.state[i] ^= theta
            
            for i in range(4):
                a = self.state[i * 2]
                b = self.state[i * 2 + 1]
                self.state[i * 2] = a ^ ((~b) & self.state[(i * 2 + 2) % 8])
                self.state[i * 2 + 1] = b ^ ((~self.state[(i * 2 + 2) % 8]) & self.state[(i * 2 + 3) % 8])
                self.state[i * 2] &= 0xffffffffffffffff
                self.state[i * 2 + 1] &= 0xffffffffffffffff
            
            self.state[0] ^= r ^ (r << 8) ^ (r << 16)
            self.state[0] &= 0xffffffffffffffff
    
    def squeeze(self, length):
        output = bytearray(length)
        for i in range(length):
            if i % 64 == 0 and i > 0:
                self.permute()
            output[i] = (self.state[(i // 8) % 8] >> ((i % 8) * 8)) & 0xff
        return bytes(output)
    
    def get_state(self, idx):
        return self.state[idx % 8]


class chaosengine:
    def __init__(self, seed):
        self.x = ((seed & 0xffff) / 65535.0) * 20.0 - 10.0
        self.y = (((seed >> 16) & 0xffff) / 65535.0) * 20.0 - 10.0
        self.z = (((seed >> 32) & 0xffff) / 65535.0) * 30.0 + 10.0
        self.sigma = 10.0
        self.rho = 28.0
        self.beta = 8.0 / 3.0
        self.dt = 0.001
    
    def step(self):
        dx = self.sigma * (self.y - self.x)
        dy = self.x * (self.rho - self.z) - self.y
        dz = self.x * self.y - self.beta * self.z
        
        self.x += dx * self.dt
        self.y += dy * self.dt
        self.z += dz * self.dt
    
    def generate(self):
        result = 0
        for i in range(64):
            self.step()
            bit = int((self.x + self.y + self.z) * 1000000.0) & 1
            result |= bit << i
        return result
    
    def generate_byte(self):
        result = 0
        for i in range(8):
            for _ in range(10):
                self.step()
            bit = int((self.x * self.y * self.z) * 1000000.0) & 1
            result |= bit << i
        return result


class nonlinearfeedback:
    def __init__(self, seed1, seed2, seed3):
        self.lfsr1 = seed1 | 1
        self.lfsr2 = seed2 | 1
        self.lfsr3 = seed3 | 1
        self.nlfsr = seed1 ^ seed2 ^ seed3
    
    def clock(self):
        bit1 = ((self.lfsr1 >> 0) ^ (self.lfsr1 >> 2) ^ (self.lfsr1 >> 3) ^ (self.lfsr1 >> 5)) & 1
        self.lfsr1 = ((self.lfsr1 >> 1) | (bit1 << 63)) & 0xffffffffffffffff
        
        bit2 = ((self.lfsr2 >> 0) ^ (self.lfsr2 >> 1) ^ (self.lfsr2 >> 2) ^ (self.lfsr2 >> 7)) & 1
        self.lfsr2 = ((self.lfsr2 >> 1) | (bit2 << 63)) & 0xffffffffffffffff
        
        bit3 = ((self.lfsr3 >> 0) ^ (self.lfsr3 >> 1) ^ (self.lfsr3 >> 5) ^ (self.lfsr3 >> 6)) & 1
        self.lfsr3 = ((self.lfsr3 >> 1) | (bit3 << 63)) & 0xffffffffffffffff
        
        maj = (self.lfsr1 & self.lfsr2) ^ (self.lfsr2 & self.lfsr3) ^ (self.lfsr1 & self.lfsr3)
        
        self.nlfsr ^= (self.lfsr1 & self.lfsr2 & self.lfsr3) | ((~self.lfsr1) & (self.lfsr2 ^ self.lfsr3))
        self.nlfsr = (((self.nlfsr << 1) | (self.nlfsr >> 63)) ^ maj) & 0xffffffffffffffff
        
        return (self.nlfsr ^ self.lfsr1 ^ self.lfsr2 ^ self.lfsr3) & 0xffffffffffffffff


class dynamicsbox:
    def __init__(self, key):
        self.forward = list(range(256))
        self.inverse = list(range(256))
        self._generate(key)
    
    def _generate(self, key):
        qs = quantumstate()
        qs.absorb(key)
        
        ce = chaosengine(qs.get_state(0))
        nf = nonlinearfeedback(qs.get_state(1), qs.get_state(2), qs.get_state(3))
        
        for _ in range(32):
            for i in range(255, 0, -1):
                r1 = ce.generate()
                r2 = nf.clock()
                j = (r1 ^ r2) % (i + 1)
                self.forward[i], self.forward[j] = self.forward[j], self.forward[i]
            qs.permute()
        
        for i in range(256):
            self.inverse[self.forward[i]] = i
    
    def substitute(self, data):
        return bytes([self.forward[b] for b in data])
    
    def inverse_substitute(self, data):
        return bytes([self.inverse[b] for b in data])


class contextboundkey:
    def __init__(self):
        pass
    
    def get_context(self):
        context = bytearray()
        
        context.extend(struct.pack("<Q", int(time.time() * 1000000)))
        
        context.extend(platform.system().encode()[:8].ljust(8, b'\x00'))
        
        context.extend(platform.machine().encode()[:8].ljust(8, b'\x00'))
        
        context.extend(struct.pack("<Q", sys.maxsize))
        
        context.extend(struct.pack("<I", os.getpid()))
        
        context.extend(os.urandom(16))
        
        return bytes(context)
    
    def derive_bound_key(self, master_key, context=None):
        if context is None:
            context = self.get_context()
        
        qs = quantumstate()
        qs.absorb(master_key)
        qs.absorb(context)
        
        ce = chaosengine(qs.get_state(0) ^ qs.get_state(4))
        nf = nonlinearfeedback(qs.get_state(1), qs.get_state(2), qs.get_state(3))
        
        output = bytearray(len(master_key))
        for i in range(len(master_key)):
            qs.permute()
            q_byte = qs.get_state(i % 8) & 0xff
            c_byte = ce.generate_byte()
            n_byte = nf.clock() & 0xff
            output[i] = q_byte ^ c_byte ^ n_byte ^ master_key[i % len(master_key)]
        
        return bytes(output), context


class obsidian3300:
    block_bits = 3296
    block_size = block_bits // 8
    key_size = block_size
    default_rounds = 32
    
    def __init__(self, key=None, context_bound=False):
        if key is None:
            key = os.urandom(self.key_size)
        
        if len(key) < self.key_size:
            key = self._expand_key(key, self.key_size)
        
        self._master_key = key[:self.key_size]
        self._context_bound = context_bound
        self._context = None
        
        if context_bound:
            cb = contextboundkey()
            self._master_key, self._context = cb.derive_bound_key(self._master_key)
        
        self._sbox = dynamicsbox(self._master_key)
        self._iv = os.urandom(self.block_size)
        self._round_keys = self._generate_round_keys()
    
    def _expand_key(self, key, target_len):
        qs = quantumstate()
        qs.absorb(key)
        return qs.squeeze(target_len)
    
    def _generate_round_keys(self):
        round_keys = []
        qs = quantumstate()
        qs.absorb(self._master_key)
        
        for r in range(self.default_rounds):
            qs.permute()
            rk = qs.squeeze(self.key_size)
            
            ce = chaosengine(qs.get_state(r % 8))
            nf = nonlinearfeedback(qs.get_state(0), qs.get_state(1), qs.get_state(2))
            
            mixed = bytearray(self.key_size)
            for i in range(self.key_size):
                mixed[i] = rk[i] ^ ce.generate_byte() ^ (nf.clock() & 0xff)
            
            round_keys.append(bytes(mixed))
        
        return round_keys
    
    def _feistel_round(self, left, right, round_key):
        qs = quantumstate()
        qs.absorb(right)
        qs.absorb(round_key)
        
        f = qs.squeeze(len(left))
        f = self._sbox.substitute(f)
        
        new_left = right
        new_right = bytes([l ^ fv for l, fv in zip(left, f)])
        
        return new_left, new_right
    
    def _inverse_feistel_round(self, left, right, round_key):
        qs = quantumstate()
        qs.absorb(left)
        qs.absorb(round_key)
        
        f = qs.squeeze(len(right))
        f = self._sbox.substitute(f)
        
        new_right = left
        new_left = bytes([r ^ fv for r, fv in zip(right, f)])
        
        return new_left, new_right
    
    def _mixing_layer(self, data, round_key):
        result = bytearray(data)
        
        for i in range(len(result)):
            result[i] ^= round_key[i % len(round_key)]
        
        for i in range(len(result) - 1):
            result[i + 1] ^= result[i]
        
        return bytes(result)
    
    def _inverse_mixing_layer(self, data, round_key):
        result = bytearray(data)
        
        for i in range(len(result) - 2, -1, -1):
            result[i + 1] ^= result[i]
        
        for i in range(len(result)):
            result[i] ^= round_key[i % len(round_key)]
        
        return bytes(result)
    
    def _encrypt_block(self, block):
        half = len(block) // 2
        left = block[:half]
        right = block[half:]
        
        for r in range(self.default_rounds):
            left = self._sbox.substitute(left)
            right = self._sbox.substitute(right)
            
            left, right = self._feistel_round(left, right, self._round_keys[r])
            
            combined = bytes(left) + bytes(right)
            combined = self._mixing_layer(combined, self._round_keys[r])
            left = combined[:half]
            right = combined[half:]
        
        return bytes(left) + bytes(right)
    
    def _decrypt_block(self, block):
        half = len(block) // 2
        left = block[:half]
        right = block[half:]
        
        for r in range(self.default_rounds - 1, -1, -1):
            combined = bytes(left) + bytes(right)
            combined = self._inverse_mixing_layer(combined, self._round_keys[r])
            left = combined[:half]
            right = combined[half:]
            
            left, right = self._inverse_feistel_round(left, right, self._round_keys[r])
            
            left = self._sbox.inverse_substitute(left)
            right = self._sbox.inverse_substitute(right)
        
        return bytes(left) + bytes(right)
    
    def _pad(self, data):
        pad_len = self.block_size - (len(data) % self.block_size)
        if pad_len == 0:
            pad_len = self.block_size
        
        if pad_len < 4:
            pad_len += self.block_size
        
        length_bytes = struct.pack(">I", pad_len)
        pad_byte = (pad_len & 0xff) if pad_len > 4 else 0x80
        padding = bytes([pad_byte] * (pad_len - 4)) + length_bytes
        return data + padding
    
    def _unpad(self, data):
        length_bytes = data[-4:]
        pad_len = struct.unpack(">I", length_bytes)[0]
        return data[:-pad_len]
    
    def _xor_bytes(self, a, b):
        return bytes([x ^ y for x, y in zip(a, b)])
    
    def encrypt(self, plaintext):
        if isinstance(plaintext, str):
            plaintext = plaintext.encode()
        
        padded = self._pad(plaintext)
        blocks = [padded[i:i + self.block_size] for i in range(0, len(padded), self.block_size)]
        
        encrypted = []
        prev = self._iv
        
        for block in blocks:
            xored = self._xor_bytes(block, prev)
            enc = self._encrypt_block(xored)
            encrypted.append(enc)
            prev = enc
        
        return b"".join(encrypted)
    
    def decrypt(self, ciphertext):
        blocks = [ciphertext[i:i + self.block_size] for i in range(0, len(ciphertext), self.block_size)]
        
        decrypted = []
        prev = self._iv
        
        for block in blocks:
            dec = self._decrypt_block(block)
            xored = self._xor_bytes(dec, prev)
            decrypted.append(xored)
            prev = block
        
        padded = b"".join(decrypted)
        return self._unpad(padded)
    
    def encrypt_with_iv(self, plaintext):
        ciphertext = self.encrypt(plaintext)
        return self._iv + ciphertext
    
    def decrypt_with_iv(self, data):
        self._iv = data[:self.block_size]
        ciphertext = data[self.block_size:]
        return self.decrypt(ciphertext)
    
    def get_key(self):
        return self._master_key
    
    def get_iv(self):
        return self._iv
    
    def get_context(self):
        return self._context



from .sbox import obsidiansbox
from .permutation import obsidianpermutation
from .mixer import obsidianmixer
from .keygen import keygenerator
from .utils import pad_data, unpad_data, xor_bytes, split_blocks, join_blocks


class obsidian256cipher:
    block_size = 32
    key_size = 32
    
    def __init__(self, key=None, iv=None):
        self._keygen = keygenerator(key_size=self.key_size)
        self._master_key = key if key else self._keygen.generate_master_key()
        self._iv = iv if iv else self._keygen.generate_iv()
        self._round_keys = None
        self._sboxes = []
        self._permutations = []
        self._mixer = obsidianmixer(self.block_size)
        self._initialized = False
        self._decoy_operations = True
    
    def _initialize(self):
        if self._initialized:
            return
        
        self._round_keys = self._keygen.generate_round_keys(self._master_key)
        num_rounds = len(self._round_keys)
        
        self._sboxes = []
        self._permutations = []
        
        for r in range(num_rounds):
            seed = int.from_bytes(self._round_keys[r][:8], "little")
            sbox = obsidiansbox(seed)
            self._sboxes.append(sbox)
            
            perm = obsidianpermutation(self.block_size)
            perm.generate_permutation(seed ^ 0xdeadbeef)
            self._permutations.append(perm)
        
        self._initialized = True
    
    def _decoy_round(self, data, round_num):
        if not self._decoy_operations:
            return data
        
        temp = bytearray(data)
        for i in range(len(temp)):
            temp[i] ^= (round_num * 0x37) & 0xff
        
        for i in range(len(temp)):
            temp[i] ^= (round_num * 0x37) & 0xff
        
        return bytes(temp)
    
    def _encrypt_block(self, block):
        self._initialize()
        
        if len(block) != self.block_size:
            raise ValueError(f"block must be {self.block_size} bytes")
        
        state = block
        num_rounds = len(self._round_keys)
        
        state = xor_bytes(state, self._master_key)
        
        for r in range(num_rounds):
            state = self._decoy_round(state, r)
            
            state = self._sboxes[r].substitute(state)
            
            state = self._permutations[r].permute(state)
            
            state = self._mixer.mix_forward(state, self._round_keys[r])
            
            state = xor_bytes(state, self._round_keys[r])
            
            if r % 2 == 0:
                state = self._decoy_round(state, r + 100)
        
        state = xor_bytes(state, self._master_key)
        
        return state
    
    def _decrypt_block(self, block):
        self._initialize()
        
        if len(block) != self.block_size:
            raise ValueError(f"block must be {self.block_size} bytes")
        
        state = block
        num_rounds = len(self._round_keys)
        
        state = xor_bytes(state, self._master_key)
        
        for r in range(num_rounds - 1, -1, -1):
            if r % 2 == 0:
                state = self._decoy_round(state, r + 100)
            
            state = xor_bytes(state, self._round_keys[r])
            
            state = self._mixer.mix_inverse(state, self._round_keys[r])
            
            state = self._permutations[r].inverse_permute(state)
            
            state = self._sboxes[r].inverse_substitute(state)
            
            state = self._decoy_round(state, r)
        
        state = xor_bytes(state, self._master_key)
        
        return state
    
    def encrypt(self, plaintext):
        if isinstance(plaintext, str):
            plaintext = plaintext.encode("utf-8")
        
        padded = pad_data(plaintext, self.block_size)
        blocks = split_blocks(padded, self.block_size)
        
        encrypted_blocks = []
        prev_block = self._iv
        
        for block in blocks:
            xored = xor_bytes(block, prev_block)
            encrypted = self._encrypt_block(xored)
            encrypted_blocks.append(encrypted)
            prev_block = encrypted
        
        return join_blocks(encrypted_blocks)
    
    def decrypt(self, ciphertext):
        if len(ciphertext) % self.block_size != 0:
            raise ValueError("invalid ciphertext length")
        
        blocks = split_blocks(ciphertext, self.block_size)
        
        decrypted_blocks = []
        prev_block = self._iv
        
        for block in blocks:
            decrypted = self._decrypt_block(block)
            xored = xor_bytes(decrypted, prev_block)
            decrypted_blocks.append(xored)
            prev_block = block
        
        padded = join_blocks(decrypted_blocks)
        return unpad_data(padded)
    
    def encrypt_with_iv(self, plaintext):
        ciphertext = self.encrypt(plaintext)
        return self._iv + ciphertext
    
    def decrypt_with_iv(self, data):
        if len(data) < self.block_size:
            raise ValueError("data too short")
        iv = data[:self.block_size]
        ciphertext = data[self.block_size:]
        self._iv = iv
        return self.decrypt(ciphertext)
    
    def get_key(self):
        return self._master_key
    
    def get_iv(self):
        return self._iv
    
    def get_round_count(self):
        self._initialize()
        return len(self._round_keys)
    
    def set_decoy_operations(self, enabled):
        self._decoy_operations = enabled
    
    @classmethod
    def generate_key(cls):
        kg = keygenerator(key_size=cls.key_size)
        return kg.generate_master_key()
    
    @classmethod
    def generate_iv(cls):
        kg = keygenerator(key_size=cls.key_size)
        return kg.generate_iv()
